;;;; "Advanced Choreography" -- this is the final video for this project.

(ns com.aurellem.run.adv-choreo
  (:use (com.aurellem.gb saves gb-driver util constants
                         items vbm characters money
                         rlm-assembly))
  (:use (com.aurellem.run util music title save-corruption
                          bootstrap-0 bootstrap-1 image
                          ram-display final-cut basic-choreo))
  (:require clojure.string)
  (:import [com.aurellem.gb.gb_driver SaveState])
  (:import java.io.File))



;; Use the gameboy's screen to display the new programming
;; instead of a side window. This will make it look much
;; cooler and create a terminal-like effect as the game is
;; being reprogramed. To do this, use a fixed data entry
;; region in ram, and run a program that translates this
;; region into the screen. Every time this data entry region
;; is full, run a program that copies the data to the
;; appropriate region in memory. This will cost ~15 seconds
;; at the beginning to set up, and then should have minimal
;; overhead (~5%) for the rest of the data transfer, but
;; will have a good psychological effect for the viewer
;; since he can see that something is actually happening in
;; the game.


;; Symbol size and type.

;; use fonts from zophar's domain:
;; http://www.zophar.net/utilities/fonts/8x8-font-archive.html

;; Green font on black background for matrix look.



;; [ ] get single long ram write.




(defn program-data [base-address]
  (let [image-program
        (display-image-kernel
         base-address
         pinkie-pie-mark)

        music-base-address (+ (count image-program) base-address)

        initial-music-data
        (midi-bytes pony-csv 0 0 0 0)
        
        data-lengths
        (map (comp count :data)
                   [(:kernel  initial-music-data)
                    (:voice-1 initial-music-data)
                    (:voice-2 initial-music-data)]);; noise not needed
        addresses
        (map (partial + music-base-address) (reductions + 0 data-lengths))

        final-music-data
        (apply (partial midi-bytes pony-csv) addresses)

        music-program
        (concat
         (:data (:kernel  final-music-data))
         (:data (:voice-1 final-music-data))
         (:data (:voice-2 final-music-data))
         (:data (:noise   final-music-data)))]

    (concat
     image-program ;; image program falls through to music program
     music-program)))



(def main-program-base-address 0xC000)

(defn write-all-program-data
  ([] (write-all-program-data (silence-noise)))
  ([script]
     (let [base-address main-program-base-address]
       (->> script
            (write-RAM base-address (program-data base-address))))))

(defn activate-program
  ([] (activate-program (write-all-program-data)))
  ([script]
     (->> script
          (transfer-control main-program-base-address)
          (do-nothing 1800))))
     